package adminkindboxhandler

import (
	"net/http"

	params "git.gocasts.ir/ebhomengo/niki/param"
	param "git.gocasts.ir/ebhomengo/niki/param/admin/kind_box"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	"github.com/labstack/echo/v4"
)

// AssignReceiverAgent godoc
// @Summary      Admin assign receiver agent to kindbox
// @Tags         KindBox
// @Accept       json
// @Produce      json
// @Param 		 id path int true "KindBox ID"
// @Param 		 Request body param.AssignReceiverRequest true "Assign Receiver Agent Request Body"
// @Success      204
// @Failure      400  {string}  "Bad request"
// @Security 	 AuthBearerAdmin
// @Router       /admin/kindboxes/assign-receiver-agent/{id} [patch].
func (h Handler) AssignReceiverAgent(c echo.Context) error {
	var req param.AssignReceiverRequest

	if bErr := c.Bind(&req); bErr != nil {
		return echo.NewHTTPError(http.StatusBadRequest)
	}

	resp, sErr := h.adminKindBoxSvc.AssignReceiverAgent(c.Request().Context(), req)

	if sErr != nil {
		msg, code := httpmsg.Error(sErr)
		if resp.FieldErrors != nil {
			return c.JSON(code, echo.Map{
				"message": msg,
				"errors":  resp.FieldErrors,
			})
		}

		return echo.NewHTTPError(code, msg)
	}

	go h.notificationSvc.KindBoxAssigned(params.NotificationKindBoxAssigned{
		ReceiverAgentID: req.ReceiverAgentID,
	})

	return c.JSON(http.StatusNoContent, nil)
}
